/*
    File:       serialdac.cpp
    Version:    0.1.0
    Date:       Apr 7, 2015
	License:	GPL v2
    
	Cpp file for serial dac using a 74HCT164 serial in, parallel
	out shift register and a 8Bit R/2R network
	 
    ****************************************************************************
    Copyright (C) 2015 Radu Motisan  <radu.motisan@gmail.com>

	http://www.pocketmagic.net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ****************************************************************************
 */


#include "serialdac.h"
#include <stdio.h>

void serialdac::init(volatile uint8_t  *portData, uint8_t  dqData, volatile uint8_t  *portCLK, uint8_t dqCLK) {
	// save globals
	m_pportRsData = portData;
	m_dqrsdata = dqData;
	m_pportCLK = portCLK;
	m_dqclk = dqCLK;
	
	// init data and clk ports
	*m_pportRsData &= ~(1<<m_dqrsdata); //set pins to LOW
	*m_pportCLK &= ~(1<<m_dqclk); //set pins to LOW
	
	*Port2DDR(m_pportRsData) |= (1<<m_dqrsdata);//Enable pins as outputs
	*Port2DDR(m_pportCLK) |= (1<<m_dqclk);//Enable pins as outputs
}

/*
 * Loads a byte into the shift register (74'164).
 *
 * Parameters:
 *      b        The byte to load into the '164.
*/
void serialdac::sendbyte(uint8_t b)
{
	// make sure clock is low 
	*m_pportCLK &= ~(1<<m_dqclk);
	
	for(int i=0; i<8; i++) {
		// loop through bits 	
		if (b & 0x80) 
			*m_pportRsData |=(1<<m_dqrsdata); // this bit is high 
		else 
			*m_pportRsData &= ~(1<<m_dqrsdata); //this bit is low 
		// next bit
		b = b << 1;
		// pulse the the shift register clock *
		*m_pportCLK |= (1<<m_dqclk);
		*m_pportCLK &= ~(1<<m_dqclk);
	}
}

